/**********************************************************************
 *
 * This code is part of the OOML project
 * Authors: Nieves Cubo
 *
 * OOML is licenced under the Common Creative License,
 * Attribution-ShareAlike 3.0
 *
 * You are free:
 *   - to Share - to copy, distribute and transmit the work
 *   - to Remix - to adapt the work
 *
 * Under the following conditions:
 *   - Attribution. You must attribute the work in the manner specified
 *     by the author or licensor (but not in any way that suggests that
 *     they endorse you or your use of the work).
 *   - Share Alike. If you alter, transform, or build upon this work,
 *     you may distribute the resulting work only under the same or
 *     similar license to this one.
 *
 * Any of the above conditions can be waived if you get permission
 * from the copyright holder.  Nothing in this license impairs or
 * restricts the author's moral rights.
 *
 * It is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied
 * warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR
 * PURPOSE.
 **********************************************************************/

#include <components/Cube.h>
#include <components/Cylinder.h>
#include <core/IndentWriter.h>
#include <core/Difference.h>
#include <core/Union.h>
#include <components/PolygonalPrism.h>
#include <core/Polygon2D.h>
#include <core/Intersection.h>
#include <math.h>
#include <core/Component2D.h>
#include <core/Point2D.h>

#include <iostream>
#include <fstream>

#define PI 3.14159

using namespace std;


int main(int argc, char **argv)
{
    IndentWriter writer;

    ofstream os("PentacleWheel.scad");


    //Independent Parameters

    double rExt=28,         //Wheel radius (Without O-Ring)
            width=4,        //External cylinder width
            height=5.5,       //Wheel height
            OringDiam=3;    //O-Ring diameter

    int     r=11.5,           //Radius of the circle circumscribed to the Pentagon
            x=3;            //Star width

    //Dependent Parameters. Internal parametric calculus

    double l=(1.8*rExt*cos(PI/10));


    // External cylinder
    Component Wheel =  Cylinder::create(rExt, height, 100,true) - Cylinder::create(rExt-width, height+2,100, true);

    //Internal cylinder
    Component Axe_cyl= Cylinder::create(4.15,height+2,100,true);

    //Servo hole
    Component Serv_cyl=  Cylinder::create(10.7,2,100,true).translate(0,0,height/2-0.75);

    //Central pentagon
    Polygon<Point2D> Pentagon;

    Pentagon.addPoint(Point2D(0,r*sin(PI/2)));
    Pentagon.addPoint(Point2D(r*cos(PI/10),r*sin(PI/10)));
    Pentagon.addPoint(Point2D(r*cos(3*PI/10),-r*sin(3*PI/10)));
    Pentagon.addPoint(Point2D(-r*cos(3*PI/10),-r*sin(3*PI/10)));
    Pentagon.addPoint(Point2D(-r*cos(PI/10),r*sin(PI/10)));

    Component Pentah = PolygonalPrism(Pentagon, height).translate(0,0,-height/2);

    Component Center = (Pentah - Axe_cyl);


    //Star - Pentacle
    Component Star = Cube::create(x,l,height).rotate(0,0,18).translate(-r*cos(PI/10)+x,-r*sin(PI/10),0);

    for(int i=0; i<5; i++)
    {
        Star=Star+Star.rotatedCopy(0,0,72);;
    }


    Component2D triangle(Polygon2D::create());
    {
        Polygon2D & triangle2D = triangle.getRef<Polygon2D>();

        triangle2D.addPoint(Point2D(1.5, -1.5));
        triangle2D.addPoint(Point2D(1.5, 1.5));
        triangle2D.addPoint(Point2D(0.0, 0.0));
    }

    triangle.translate(rExt-OringDiam/2,0,0);

    //O-Ring hole
    Component OringHole(triangle.rotateExtrudedCopy(10, 100));

    //Final union and difference
    Component PentacleWheel = (Wheel + Center + Star) - OringHole - Serv_cyl;


    writer << PentacleWheel;
    os << writer;
    os.close();


}

